
/*
*    FreeRTOS V6.0.5 - Copyright (C) 2010 Real Time Engineers Ltd.
* 
***************************************************************************
*                                                                         *
* If you are:                                                             *
*                                                                         *
*    + New to FreeRTOS,                                                   *
*    + Wanting to learn FreeRTOS or multitasking in general quickly       *
*    + Looking for basic training,                                        *
*    + Wanting to improve your FreeRTOS skills and productivity           *
*                                                                         *
* then take a look at the FreeRTOS eBook                                  *
*                                                                         *
*        "Using the FreeRTOS Real Time Kernel - a Practical Guide"        *
*                  http://www.FreeRTOS.org/Documentation                  *
*                                                                         *
* A pdf reference manual is also available.  Both are usually delivered   *
* to your inbox within 20 minutes to two hours when purchased between 8am *
* and 8pm GMT (although please allow up to 24 hours in case of            *
* exceptional circumstances).  Thank you for your support!                *
*                                                                         *
***************************************************************************

This file is part of the FreeRTOS distribution.

FreeRTOS is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License (version 2) as published by the
Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
***NOTE*** The exception to the GPL is included to allow you to distribute
a combined work that includes FreeRTOS without being obliged to provide the
source code for proprietary components outside of the FreeRTOS kernel.
FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
more details. You should have received a copy of the GNU General Public 
License and the FreeRTOS license exception along with FreeRTOS; if not it 
can be viewed here: http://www.freertos.org/a00114.html and also obtained 
by writing to Richard Barry, contact details for whom are available on the
FreeRTOS WEB site.

1 tab == 4 spaces!

http://www.FreeRTOS.org - Documentation, latest information, license and
contact details.

http://www.SafeRTOS.com - A version that is certified for use in safety
critical systems.

http://www.OpenRTOS.com - Commercial support, development, porting,
licensing and training services.
*/


#include <LPC17xx.h>
#include <stdlib.h>
#include <debug_frmwrk.h>
#include <lpc17xx_clkpwr.h>
#include <lpc17xx_timer.h>
#include <lpc17xx_wdt.h>

/* Scheduler includes. */
#include <FreeRTOS.h>
#include <queue.h>
#include <task.h>
#include <semphr.h>

#include <MasterTask.h>

#define debug

/*-----------------------------------------------------------*/

/* The time between cycles of the 'check' functionality (defined within the
* tick hook). */
#define mainCHECK_DELAY						( ( portTickType ) 5000 / portTICK_RATE_MS )

/*-----------------------------------------------------------*/
void prvSetupHardware( void );

void DumpMem(void);

void LEDTask1(void);
void LEDTask2(void);

xTaskHandle xLEDTask1_TaskHandle;
xTaskHandle xLEDTask2_TaskHandle;

xSemaphoreHandle LED1Sem;
xSemaphoreHandle LED2Sem;

uint8_t UARTBuffer[50];

/*-----------------------------------------------------------*/
int main( void )
{
  
  NVIC_SetPriorityGrouping(0);
  prvSetupHardware();

  //Create Tasks:
  #ifdef debug
  UARTPuts_(LPC_UART0,"Creating Tasks Now");
  #endif

  xTaskCreate(MasterTask,"Master",100,NULL,3,NULL);
  xTaskCreate(LEDTask1,"LED1",100,NULL,2,&xLEDTask1_TaskHandle);
  xTaskCreate(LEDTask2,"LED2",100,NULL,1,&xLEDTask2_TaskHandle);

  SCB->SHCSR=(1<<18)|(1<<17)|(1<<16);		//Enable all Fault Handler

  //Create Semaphores:
  vSemaphoreCreateBinary(LED1Sem);
  vSemaphoreCreateBinary(LED2Sem);

  #ifdef debug
  UARTPuts_(LPC_UART0,"Starting Scheduler");
  #endif

  /* Start the scheduler. */
  vTaskStartScheduler();
  
  /* Will only get here if there was insufficient memory to create the idle
    * task.  The idle task is created within vTaskStartScheduler(). */
  for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
    LPC_GPIO0->FIOSET=(1<<1);	//Debugging processor usage: uncomment and also in ApplicationTickHook 
  static unsigned long ulTicksSinceLastDisplay = 0;
  
  /* Called from every tick interrupt as described in the comments at the top
    *	of this file.
    * 
    *	Have enough ticks passed to make it time to perform our health status
    *	check again? */
  ulTicksSinceLastDisplay++;
  if( ulTicksSinceLastDisplay >= mainCHECK_DELAY )
  {
    ulTicksSinceLastDisplay=0;
    /*    if(GPIO2->FIOPIN&(1<<10))
      *      GPIO2->FIOCLR=(1<<10);
      *    else
      *      GPIO2->FIOSET=(1<<10);*/
  }
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook(void)
{
  while(1==1)
  {
 LPC_GPIO0->FIOCLR=(1<<1);	//Debugging processor usage: uncomment and also in IdleHook
    //the average voltage on this pin is proportional to the spare processor time
  }
}
/*----------------------------------------------------------*/

char *pcGetTaskStatusMessage( void )
{
  /* Not bothered about a critical section here. */
  // return pcStatusMessage;
}
/*-----------------------------------------------------------*/

void prvSetupHardware( void )
{
  
  //-------------------------PLL Konfiguration,CPU=80MHz, ExtOsc=12MHz, For details see System_LPC17xx.c in folder Libraries->NXP->CORE->CM3->DeviceSupport->NXP->LPC17xx-------------------------------- 
  
  SystemCoreClockUpdate();
  debug_frmwrk_init();
  #ifdef debug
  UARTPuts_(LPC_UART0,"Hardware Initialized, now Launching FreeRtos");
  #endif
  LPC_GPIO0->FIODIR|=(1<<1);

}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
  /* This function will get called if a task overflows its stack. */
  
  UARTPuts_(LPC_UART0,"TaskStackOVF:");
  UARTPuts_(LPC_UART0,pcTaskName);
  ( void ) pxTask;
  ( void ) pcTaskName;
  LPC_UART0->THR='S';
  LPC_GPIO1->FIODIR=(1<<18);
  LPC_GPIO1->FIODIR=(1<<21);
  while(1==1)
  {
    LPC_GPIO1->FIOSET=(1<<18);
    LPC_GPIO1->FIOCLR=(1<<18);
    LPC_GPIO1->FIOSET=(1<<21);
    LPC_GPIO1->FIOCLR=(1<<21);
  };
}
/*-----------------------------------------------------------*/

void vConfigureTimerForRunTimeStats( void )
{
  const unsigned long TCR_COUNT_RESET = 2, CTCR_CTM_TIMER = 0x00, TCR_COUNT_ENABLE = 0x01;
  
  /* This function configures a timer that is used as the time base when
    *	collecting run time statistical information - basically the percentage
    *	of CPU time that each task is utilising.  It is called automatically when
    *	the scheduler is started (assuming configGENERATE_RUN_TIME_STATS is set
    *	to 1). */
  
  /* Power up and feed the timer. */
  LPC_SC->PCONP |= 0x02UL;
  LPC_SC->PCLKSEL0 = (LPC_SC->PCLKSEL0 & (~(0x3<<2))) | (0x01 << 2);
  
  /* Reset Timer 0 */
  LPC_TIM0->TCR = TCR_COUNT_RESET;
  
  /* Just count up. */
  LPC_TIM0->CTCR = CTCR_CTM_TIMER;
  
  /* Prescale to a frequency that is good enough to get a decent resolution,
    *	but not too fast so as to overflow all the time. */
  LPC_TIM0->PR =  ( configCPU_CLOCK_HZ / 10000UL ) - 1UL;
  
  /* Start the counter. */
  LPC_TIM0->TCR = TCR_COUNT_ENABLE;
}
/*-----------------------------------------------------------*/

void startup_delay(void)
{;
}

void Hard_Fault_Handler_C(unsigned int * hardfault_args)
{
  WDT_Init(WDT_CLKSRC_IRC,WDT_MODE_INT_ONLY);		//Setup WDT

  unsigned int stacked_r0;
unsigned int stacked_r1;
unsigned int stacked_r2;
unsigned int stacked_r3;
unsigned int stacked_r12;
unsigned int stacked_lr;
unsigned int stacked_pc;
unsigned int stacked_psr;

stacked_r0 = ((unsigned long) hardfault_args[0]);
stacked_r1 = ((unsigned long) hardfault_args[1]);
stacked_r2 = ((unsigned long) hardfault_args[2]);
stacked_r3 = ((unsigned long) hardfault_args[3]);

stacked_r12 = ((unsigned long) hardfault_args[4]);
stacked_lr = ((unsigned long) hardfault_args[5]);
stacked_pc = ((unsigned long) hardfault_args[6]);
stacked_psr = ((unsigned long) hardfault_args[7]);

sprintf (UARTBuffer,"\n \n \n \r [Hard fault handler] \n \r ");
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"R0 = %x \n \r", stacked_r0);
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"R1 = %x \n \r", stacked_r1);
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"R2 = %x \n \r", stacked_r2);
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"R3 = %x \n \r", stacked_r3);
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"R12 = %x \n \r", stacked_r12);
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"LR = %x \n \r", stacked_lr);
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"PC = %x \n \r", stacked_pc);
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"PSR = %x \n \r", stacked_psr);
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"BFAR = %x \n \r", (*((volatile unsigned long *)(0xE000ED38))));
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"CFSR = %x \n \r", (*((volatile unsigned long *)(0xE000ED28))));
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"HFSR = %x \n \r", (*((volatile unsigned long *)(0xE000ED2C))));
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"DFSR = %x \n \r", (*((volatile unsigned long *)(0xE000ED30))));
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"AFSR = %x \n \r", (*((volatile unsigned long *)(0xE000ED3C))));
UARTPuts(LPC_UART0,UARTBuffer);

  LPC_UART0->THR='H';
  DumpMem();

  while(1==1)
  {
    LPC_GPIO1->FIOSET=(1<<18)|(1<<20);
    LPC_GPIO1->FIOCLR=(1<<18)|(1<<20);
  }


}

void Mem_Manage_Handler_C(void)
{
  LPC_UART0->THR='M';
  while(1==1)
  {
    LPC_GPIO1->FIOSET=(1<<18);
    LPC_GPIO1->FIOCLR=(1<<18);
  }
}

void Bus_Fault_Handler_C(unsigned int * busfault_args)
{

WDT_Init(WDT_CLKSRC_IRC,WDT_MODE_INT_ONLY);		//Setup WDT

unsigned int stacked_r0;
unsigned int stacked_r1;
unsigned int stacked_r2;
unsigned int stacked_r3;
unsigned int stacked_r12;
unsigned int stacked_lr;
unsigned int stacked_pc;
unsigned int stacked_psr;

stacked_r0 = ((unsigned long) busfault_args[0]);
stacked_r1 = ((unsigned long) busfault_args[1]);
stacked_r2 = ((unsigned long) busfault_args[2]);
stacked_r3 = ((unsigned long) busfault_args[3]);

stacked_r12 = ((unsigned long) busfault_args[4]);
stacked_lr = ((unsigned long) busfault_args[5]);
stacked_pc = ((unsigned long) busfault_args[6]);
stacked_psr = ((unsigned long) busfault_args[7]);

sprintf (UARTBuffer,"\n \n \n \r [Bus fault handler] \n \r ");
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"R0 = %x \n \r", stacked_r0);
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"R1 = %x \n \r", stacked_r1);
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"R2 = %x \n \r", stacked_r2);
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"R3 = %x \n \r", stacked_r3);
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"R12 = %x \n \r", stacked_r12);
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"LR = %x \n \r", stacked_lr);
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"PC = %x \n \r", stacked_pc);
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"PSR = %x \n \r", stacked_psr);
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"BFAR = %x \n \r", (*((volatile unsigned long *)(0xE000ED38))));
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"CFSR = %x \n \r", (*((volatile unsigned long *)(0xE000ED28))));
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"HFSR = %x \n \r", (*((volatile unsigned long *)(0xE000ED2C))));
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"DFSR = %x \n \r", (*((volatile unsigned long *)(0xE000ED30))));
UARTPuts(LPC_UART0,UARTBuffer);
sprintf (UARTBuffer,"AFSR = %x \n \r", (*((volatile unsigned long *)(0xE000ED3C))));
UARTPuts(LPC_UART0,UARTBuffer);

LPC_UART0->THR='B';
DumpMem();

  while(1==1)
  {
    LPC_GPIO1->FIOSET=(1<<21);
    LPC_GPIO1->FIOCLR=(1<<21);
  }
}

void Usage_Fault_Handler_C(void)
{
  LPC_UART0->THR='U';
  while(1==1)
  {
    LPC_GPIO1->FIOSET=(1<<23);
    LPC_GPIO1->FIOCLR=(1<<23);
  }
}

void NMI_Handler_C(void)
{
  LPC_UART0->THR='N';
  while(1==1)
  {
    LPC_GPIO1->FIOSET=(1<<23)|(1<<20);
    LPC_GPIO1->FIOCLR=(1<<23)|(1<<20);
  }
}

void _sbrk(void)
{;}
void _write(void)
{;}
void _close(void)
{;}
void _fstat(void)
{;}
void _isatty(void)
{;}
void _lseek(void)
{;}
void _read(void)
{;}
void _init(void)
{;}


void DumpMem(void)
{
  uint8_t *Ptr;
  Ptr=0x10000000;
  while(Ptr<0x10007FFF)
  {
    Ptr++;
    uint8_t Temp=*Ptr;
    UARTPutChar (LPC_UART0,Temp);
  }
}

void LEDTask1(void)
{
  LPC_GPIO1->FIODIR|=(1<<18);
  while(1==1)
  {
    LPC_GPIO1->FIOPIN^=(1<<18);
    vTaskDelay(10);
  }
}

void LEDTask2(void)
{
  LPC_GPIO1->FIODIR|=(1<<21);
  while(1==1)
  {
    LPC_GPIO1->FIOPIN^=(1<<21);
    vTaskDelay(100);
  }
}


